@@ -107,7 +107,7 @@ def create_task_factory(task_factory=None):
         task_factory = _default_task_factory
 
     def wrapped(loop, coro):
-        parent_task = Task.current_task(loop=loop)
+        parent_task = current_task(loop=loop)
         child_task = task_factory(loop, coro)
         if child_task._source_traceback:
             del child_task._source_traceback[-1]
