@@ -4,28 +4,34 @@
 #
 #     https://www.sqreen.io/terms.html
 #
-from ....constants import ACTIONS
 from ....rules import RuleCallback
-from ....workflow import Transport
+from ....rules_actions import RecordTransportMixin
 
 
-class HttpClientTransportCallback(RuleCallback):
+class HttpClientTransportCallback(RecordTransportMixin, RuleCallback):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         try:
             host_ip, host_port = instance.sock.getpeername()
         except Exception:
             host_ip, host_port = None, None
 
-        transport = Transport({
+        host = instance.host
+        if ":" in host:
+            try:
+                host, host_port = host.split(":", 1)
+                host_port = int(host_port)
+            except Exception:
+                pass
+
+        transport = {
             "type": "http",
-            "host": instance.host or host_ip,
+            "host": host or host_ip,
             "host_ip": host_ip,
             "host_port": host_port,
-        })
+        }
 
-        context = self.runner.interface_manager.call("context")
-        if not self.runner.interface_manager.call("new_data_out_pre", context, transport):
+        if not self.record_transport("client", transport):
             return
 
         tracing_identifier = transport.get("tracing_identifier")
@@ -36,12 +42,12 @@ class HttpClientTransportCallback(RuleCallback):
         new_headers = new_args[3] = dict(args[3])
         new_headers["X-Sqreen-Trace-Identifier"] = tracing_identifier
         return {
-            "status": ACTIONS["MODIFY_ARGS"],
+            "status": "modify_args",
             "args": (new_args, kwargs),
         }
 
 
-class HttpClientTransportAdapter:
+class HttpClientTransportAdapter(object):
 
     def instrumentation_callbacks(self, runner, storage):
         return [
