@@ -5,12 +5,12 @@
 #     https://www.sqreen.io/terms.html
 #
 from ....rules import RuleCallback
-from ....workflow import Transport
+from ....rules_actions import RecordTransportMixin
 
 
-class MySQLDbTransportCallback(RuleCallback):
+class MySQLDbTransportCallback(RecordTransportMixin, RuleCallback):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         try:
             host = kwargs.get("host")
             if host is None:
@@ -22,17 +22,15 @@ class MySQLDbTransportCallback(RuleCallback):
         except Exception:
             host, port = None, None
 
-        transport = Transport({
+        transport = {
             "type": "mysql",
             "host": host,
             "host_port": port,
-        })
-
-        context = self.runner.interface_manager.call("context")
-        self.runner.interface_manager.call("new_data_out_pre", context, transport)
+        }
+        self.record_transport("client", transport)
 
 
-class MySQLDbTransportAdapter:
+class MySQLDbTransportAdapter(object):
 
     def instrumentation_callbacks(self, runner, storage):
         return [
