@@ -142,14 +142,6 @@ class SqreenHttpException(SqreenException):
     pass
 
 
-class InvalidResponseContentType(SqreenHttpException):
-    def __init__(self, content_type):
-        self.content_type = content_type
-
-    def __repr__(self):
-        return "Invalid response Content-Type: {!r}".format(self.content_type)
-
-
 class InvalidJsonResponse(SqreenHttpException):
     def __init__(self, parsing_exception):
         self.parsing_exception = parsing_exception
