@@ -175,6 +175,10 @@ class AioHTTPRequest(BaseRequest):
         """Dictionary of cookies parameters."""
         return multidict_to_dict(self.request.cookies)
 
+    @property
+    def body(self):
+        return self.request._read_bytes
+
     @cached_property
     def _json(self):
         data = self.request._read_bytes.decode(
