@@ -15,13 +15,23 @@ from collections import defaultdict
 
 from ..config import CONFIG
 from ..remote_exception import traceback_formatter
-from ..utils import HAS_TYPING, cached_property, flatten, to_unicode_safe
+from ..utils import (
+    HAS_TYPING,
+    cached_property,
+    flatten,
+    is_unicode,
+    parse_form_params,
+    to_unicode_safe,
+)
 from .ip_utils import get_real_user_ip
 
 if sys.version_info[0] >= 3:
-    from urllib.parse import unquote_to_bytes, urlencode
+    from http.cookies import SimpleCookie
+    from urllib.parse import unquote_to_bytes
 else:
-    from urllib import unquote as unquote_to_bytes, urlencode
+    from urllib import unquote as unquote_to_bytes
+
+    from Cookie import SimpleCookie
 
 if HAS_TYPING:
     from typing import List
@@ -106,28 +116,24 @@ class BaseRequest(object):
         raise NotImplementedError
 
     @property
-    def form_params(self):
+    def query_string(self):
         raise NotImplementedError
 
-    @property
+    @cached_property
     def query_params(self):
-        raise NotImplementedError
+        raw_query = self.query_string
+        try:
+            return parse_form_params(raw_query)
+        except Exception:
+            LOGGER.debug(
+                "couldn't parse URL query %s", raw_query, exc_info=True
+            )
+            return {}
 
     @property
     def view_params(self):
         raise NotImplementedError
 
-    @property
-    def query_string(self):
-        data = []
-        for key, values in self.query_params.items():
-            if isinstance(values, list):
-                for value in values:
-                    data.append((key, value))
-            else:
-                data.append((key, values))
-        return urlencode(data)
-
     @property
     def request_uri(self):
         uri = [self.path]
@@ -136,14 +142,18 @@ class BaseRequest(object):
             uri.append("?")
             query_string = query_string.replace('+', ' ')
             # convert the query string to bytes
-            if hasattr(query_string, "encode"):
+            if sys.version_info[0] >= 3 and hasattr(query_string, "encode"):
                 query_string = query_string.encode(errors="surrogatepass")
             # unescape non-ASCII chars and try to convert the final result
             # to unicode
             uri.append(to_unicode_safe(unquote_to_bytes(query_string)))
         return "".join(uri)
 
-    @cached_property
+    @property
+    def body(self):
+        raise NotImplementedError
+
+    @property
     def json_params(self):
         try:
             ct = to_unicode_safe(self.content_type)
@@ -159,17 +169,39 @@ class BaseRequest(object):
         return {}
 
     @property
-    def cookies_params(self):
-        raise NotImplementedError
-
-    @property
-    def body(self):
-        raise NotImplementedError
+    def form_params(self):
+        try:
+            ct = to_unicode_safe(self.content_type)
+            data = self.body
+            if ct is not None and ct == "application/x-www-form-urlencoded" and data:
+                return parse_form_params(data)
+        except Exception:
+            LOGGER.debug("couldn't parse input json", exc_info=True)
+        return {}
 
     @property
     def body_params(self):
         return self.form_params or self.json_params
 
+    @property
+    def raw_cookies(self):
+        raise NotImplementedError
+
+    @property
+    def cookies_params(self):
+        cookies = self.raw_cookies
+        if cookies:
+            try:
+                cookie = SimpleCookie()
+                # Python 2 only accepts str, not unicode whereas Python 3 only accepts str, not bytes
+                if sys.version_info[0] == 2 and is_unicode(cookies):
+                    cookies = cookies.encode(errors="surrogatepass")
+                cookie.load(cookies)
+                return {key: cookie[key].coded_value for key in cookie.keys()}
+            except Exception:
+                LOGGER.debug("couldn't parse cookies", exc_info=True)
+        return {}
+
     @cached_property
     def request_uuid(self):
         return uuid.uuid4()
@@ -206,6 +238,7 @@ class BaseRequest(object):
             "scheme": self.scheme,
             "port": self.server_port,
             "remote_port": self.remote_port,
+            "endpoint": self.route,
         }
 
     @property
@@ -243,7 +276,7 @@ class BaseRequest(object):
         ret.pop("cookies", None)
         return ret
 
-    @cached_property
+    @property
     def flat_request_params(self):
         """ Flatten the request params to compute the values and keys for
         quick access
@@ -341,6 +374,10 @@ class BaseRequest(object):
             result[name] = value
         return result
 
+    @property
+    def route(self):
+        return None
+
 
 class BaseResponse(object):
 
