@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -7,13 +7,17 @@
 """ Blank request for callbacks needing a request when no one is present
 """
 
-from .base import BaseRequest
+from .base import BaseRequest, BaseResponse
 
 
 class BlankRequest(BaseRequest):
 
     @property
-    def client_ip(self):
+    def raw_headers(self):
+        return {}
+
+    @property
+    def raw_client_ip(self):
         return None
 
     @property
@@ -24,6 +28,10 @@ class BlankRequest(BaseRequest):
     def cookies_params(self):
         return {}
 
+    @property
+    def body(self):
+        return None
+
     @property
     def form_params(self):
         return {}
@@ -40,6 +48,10 @@ class BlankRequest(BaseRequest):
     def path(self):
         return None
 
+    @property
+    def query_string(self):
+        return None
+
     @property
     def query_params(self):
         return {}
@@ -56,6 +68,10 @@ class BlankRequest(BaseRequest):
     def remote_port(self):
         return None
 
+    @property
+    def remote_addr(self):
+        return None
+
     @property
     def scheme(self):
         return None
@@ -71,3 +87,18 @@ class BlankRequest(BaseRequest):
     @property
     def json_params(self):
         return {}
+
+
+class BlankResponse(BaseResponse):
+
+    @property
+    def status_code(self):
+        return None
+
+    @property
+    def content_type(self):
+        return None
+
+    @property
+    def content_length(self):
+        return None
