@@ -9,18 +9,14 @@
 import logging
 import sys
 
-from ..utils import Iterable, Mapping, cached_property, to_unicode_safe
+from ..utils import Iterable, Mapping, to_unicode_safe
 from .base import BaseRequest, BaseResponse
 
 if sys.version_info[0] >= 3:
-    from http.cookies import SimpleCookie
-    from urllib.parse import parse_qs, quote
+    from urllib.parse import quote
 else:
-    from Cookie import SimpleCookie
-    from urlparse import parse_qs
     from urllib import quote
 
-
 LOGGER = logging.getLogger(__name__)
 
 
@@ -28,41 +24,6 @@ class BaseWSGIRequest(BaseRequest):
     """ Base class for WSGI based framework requests.
     """
 
-    @property
-    def form_params(self):
-        # TODO: use a fallback form parser (like FormDataParser from werkzeug)
-        # to parse the body when form params cannot be retrieved from the
-        # framework.
-        # Returns an empty dict for now.
-        return {}
-
-    @cached_property
-    def query_params(self):
-        raw_query = self.get_raw_header("QUERY_STRING", "")
-        try:
-            return parse_qs(raw_query)
-        except Exception:
-            LOGGER.debug(
-                "couldn't parse URL query %s", raw_query, exc_info=True
-            )
-            return {}
-
-    @property
-    def view_params(self):
-        return {}
-
-    @cached_property
-    def cookies_params(self):
-        cookies = self.get_raw_header("HTTP_COOKIES")
-        if cookies:
-            try:
-                cookie = SimpleCookie()
-                cookie.load(cookies)
-                return {key: cookie[key].coded_value for key in cookie.keys()}
-            except Exception:
-                LOGGER.debug("couldn't parse cookies", exc_info=True)
-        return {}
-
     @property
     def body(self):
         from ..runtime_storage import runtime
@@ -71,6 +32,18 @@ class BaseWSGIRequest(BaseRequest):
         body, _ = storage.get_request_input_preview()
         return body
 
+    @property
+    def query_string(self):
+        return to_unicode_safe(self.get_raw_header("QUERY_STRING"))
+
+    @property
+    def view_params(self):
+        return {}
+
+    @property
+    def raw_cookies(self):
+        return to_unicode_safe(self.get_raw_header("HTTP_COOKIES"))
+
     @property
     def remote_addr(self):
         """Remote IP address."""
@@ -86,10 +59,6 @@ class BaseWSGIRequest(BaseRequest):
             self.get_raw_header("PATH_INFO", "")
         )
 
-    @property
-    def query_string(self):
-        return to_unicode_safe(self.get_raw_header("QUERY_STRING"))
-
     @property
     def method(self):
         return to_unicode_safe(self.get_raw_header("REQUEST_METHOD"))
