@@ -11,138 +11,157 @@ import logging
 import sys
 
 from .._vendors.wrapt import FunctionWrapper
-from ..constants import ACTIONS, LIFECYCLE_METHODS, VALID_ACTIONS_PER_LIFECYCLE
 from ..exceptions import ActionBlock, ActionRedirect, AttackBlocked
-from ..execute_callbacks import execute_callbacks, valid_args
+from ..execute_callbacks import execute_callbacks
 from ..runtime_storage import runtime
 from .helpers import guard_call
 
 LOGGER = logging.getLogger(__name__)
 
 
+# Order of valid actions is important, if multiple actions are returned on the
+# same hookpoint, the smallest index wins.
+VALID_ACTIONS_PRE = (
+    "raise",
+    "action_block",
+    "action_redirect",
+    "override",
+    "modify_args",
+)
+
+VALID_ACTIONS_FAILING = (
+    "raise",
+    "retry",
+    "override",
+)
+
+VALID_ACTIONS_POST = (
+    "raise",
+    "override"
+)
+
+
 def execute_pre_callbacks(
-    key, strategy, instance, args=None, kwargs=None, storage=runtime,
-    override_budget=None
+    key, callbacks, instance, args, kwargs, options, storage=runtime,
+    override_budget=None, valid_actions=VALID_ACTIONS_PRE,
 ):
     """ Execute pre_callbacks. Pre callbacks will receive these arguments:
     (instance, args, kwargs)
     """
-    pre_callbacks = strategy.get_pre_callbacks(key)
-    if pre_callbacks:
+    if callbacks:
         return guard_call(
             key,
             execute_callbacks,
-            strategy.queue,
-            pre_callbacks,
-            LIFECYCLE_METHODS["PRE"],
+            callbacks,
+            "pre",
             instance,
             args,
             kwargs,
+            options,
             storage=storage,
             override_budget=override_budget,
-            valid_actions=VALID_ACTIONS_PER_LIFECYCLE[LIFECYCLE_METHODS["PRE"]],
+            valid_actions=valid_actions,
         )
     return {}
 
 
 def execute_failing_callbacks(
-    key, strategy, instance, exc_info, args=None, kwargs=None, storage=runtime,
-    override_budget=None
+    key, callbacks, instance, args, kwargs, options, storage=runtime,
+    override_budget=None, valid_actions=VALID_ACTIONS_FAILING,
 ):
     """ Execute failing_callbacks. Failing callbacks will receive these arguments:
-    (instance, args, kwargs, exc_info=exc_info)
+    (instance, args, kwargs, options)
     """
-    failing_callbacks = strategy.get_failing_callbacks(key)
-    if failing_callbacks:
+    if callbacks:
         return guard_call(
             key,
             execute_callbacks,
-            strategy.queue,
-            failing_callbacks,
-            LIFECYCLE_METHODS["FAILING"],
+            reversed(callbacks),
+            "failing",
             instance,
             args,
             kwargs,
-            exc_info=exc_info,
+            options,
             storage=storage,
             override_budget=override_budget,
-            valid_actions=VALID_ACTIONS_PER_LIFECYCLE[LIFECYCLE_METHODS["FAILING"]],
+            valid_actions=valid_actions,
         )
     return {}
 
 
 def execute_post_callbacks(
-    key, strategy, instance, result, args=None, kwargs=None, storage=runtime,
-    override_budget=None
+    key, callbacks, instance, args, kwargs, options, storage=runtime,
+    override_budget=None, valid_actions=VALID_ACTIONS_POST,
 ):
     """ Execute post_callbacks. Post callbacks will receive these arguments:
-    (instance, args, kwargs, result=result)
+    (instance, args, kwargs, options)
     """
-    post_callbacks = strategy.get_post_callbacks(key)
-    if post_callbacks:
+    if callbacks:
         return guard_call(
             key,
             execute_callbacks,
-            strategy.queue,
-            post_callbacks,
-            LIFECYCLE_METHODS["POST"],
+            reversed(callbacks),
+            "post",
             instance,
             args,
             kwargs,
-            result=result,
+            options,
             storage=storage,
             override_budget=override_budget,
-            valid_actions=VALID_ACTIONS_PER_LIFECYCLE[LIFECYCLE_METHODS["POST"]],
+            valid_actions=valid_actions,
         )
     return {}
 
 
 def hook_point_wrapper(strategy, hook_name, hook_method, storage=runtime):
     """ Execute the original method and pre/post/failing callbacks
-    If an exception happens, create a RemoteException, call
-    callback.exception_infos for more debugging infos and send it via
-    the provided queue.
     """
     key = (hook_name, hook_method)
 
     def wrapper(wrapped, instance, args, kwargs):
         strategy.before_hook_point()
+        callbacks = strategy.settings.get_callbacks(key)
+        options = {}
 
         # Call pre callbacks
         action = execute_pre_callbacks(
             key,
-            strategy,
+            callbacks["pre"],
             instance,
             args,
             kwargs,
+            options,
             storage=storage
         )
 
-        if action.get("status") == ACTIONS["RAISE"]:
+        status = action.get("status")
+        if status is None:
+            pass
+        elif status == "modify_args":
+            args, kwargs = action["args"]
+        elif status == "override":
+            return action.get("new_return_value")
+        elif status == "raise":
             LOGGER.debug(
                 "Callback %s detected an attack", action.get("rule_name")
             )
             raise AttackBlocked(action.get("rule_name"))
-        elif action.get("status") == ACTIONS["ACTION_BLOCK"]:
+        elif status == "action_block":
             LOGGER.debug(
                 "Action %s blocked the request", action.get("action_id")
             )
             raise ActionBlock(action.get("action_id"))
-        elif action.get("status") == ACTIONS["ACTION_REDIRECT"]:
+        elif status == "action_redirect":
             LOGGER.debug(
                 "Action %s redirected the request to %r",
                 action.get("action_id"),
                 action["target_url"],
             )
             raise ActionRedirect(action.get("action_id"), action["target_url"])
-        elif action.get("status") == ACTIONS["OVERRIDE"]:
-            return action.get("new_return_value")
-        elif action.get("status") == ACTIONS["MODIFY_ARGS"]:
-            if valid_args(action["args"]):
-                args, kwargs = action["args"]
 
         # Call original method
         retry = True
+        result = None
         while retry is True:
             try:
                 retry = False
@@ -150,24 +169,27 @@ def hook_point_wrapper(strategy, hook_name, hook_method, storage=runtime):
                 result = wrapped(*args, **kwargs)
             except Exception:
                 # In case of error, call fail callbacks with exception infos
-                exc_info = sys.exc_info()
+                options["exc_info"] = sys.exc_info()
 
                 # Either raise an exception, set a return value or retry
                 action = execute_failing_callbacks(
-                    key, strategy, instance, exc_info, args, kwargs,
-                    storage=storage
+                    key, callbacks["failing"], instance, args, kwargs,
+                    options, storage=storage
                 )
 
-                if action.get("status") == ACTIONS["RAISE"]:
+                status = action.get("status")
+                if status is None:
+                    pass
+                elif status == "override":
+                    return action.get("new_return_value")
+                elif status == "raise":
                     LOGGER.debug(
                         "Callback %s detected an attack",
                         action.get("rule_name"),
                     )
                     raise AttackBlocked(action.get("rule_name"))
-                elif action.get("status") == ACTIONS["RETRY"]:
+                elif status == "retry":
                     retry = True
-                elif action.get("status") == ACTIONS["OVERRIDE"]:
-                    return action.get("new_return_value")
 
                 # Be sure to raise if no retry or override
                 if retry is False:
@@ -175,17 +197,22 @@ def hook_point_wrapper(strategy, hook_name, hook_method, storage=runtime):
 
         # Then call post callback in reverse order to simulate decorator
         # behavior
+        options["result"] = result
         action = execute_post_callbacks(
-            key, strategy, instance, result, args, kwargs, storage=storage,
+            key, callbacks["post"], instance, args, kwargs,
+            options, storage=storage,
         )
 
-        if action.get("status") == ACTIONS["RAISE"]:
+        status = action.get("status")
+        if status is None:
+            pass
+        elif status == "override":
+            return action.get("new_return_value")
+        elif status == "raise":
             LOGGER.debug(
                 "Callback %s detected an attack", action.get("rule_name")
             )
             raise AttackBlocked(action.get("rule_name"))
-        elif action.get("status") == ACTIONS["OVERRIDE"]:
-            return action.get("new_return_value")
 
         # And return the original value
         return result
