@@ -7,7 +7,6 @@
 """Middleware class for aiohttp."""
 
 import sys
-from asyncio import coroutine
 from logging import getLogger
 
 from ..._vendors.wrapt import FunctionWrapper
@@ -33,8 +32,7 @@ class AioHTTPMiddleware(BaseMiddleware):
     def __call__(self, original):
         """Wrap the original aiohttp middleware with callbacks."""
         @aiohttp_web_middleware
-        @coroutine
-        def wrapper(wrapped, instance, args, kwargs):
+        async def wrapper(wrapped, instance, args, kwargs):
             from aiohttp import web  # type: ignore
 
             request = instance
@@ -42,7 +40,7 @@ class AioHTTPMiddleware(BaseMiddleware):
             transaction = self.create_transaction()
             try:
                 self.execute_pre_callbacks(transaction, (request,), record_attack=True)
-                response = yield from wrapped(*args, **kwargs)
+                response = await wrapped(*args, **kwargs)
             except web.HTTPException as exc:
                 # Each HTTP status code is a subclass of HTTPException.
                 # These exceptions are also a subclass of Response.
@@ -63,15 +61,13 @@ class AioHTTPMiddleware(BaseMiddleware):
 
     @staticmethod
     @aiohttp_web_middleware
-    @coroutine
-    def handle(request, handler):
+    async def handle(request, handler):
         """Instrumentation point, do nothing."""
-        resp = yield from handler(request)
+        resp = await handler(request)
         return resp
 
     @staticmethod
-    @coroutine
-    def factory(app, handler):
+    async def factory(app, handler):
         """Old style middleware factory, used in aiohttp 2.2."""
 
         # This function is a coroutine (direct call to another coroutine
