@@ -7,7 +7,8 @@
 import time
 from datetime import datetime
 from logging import getLogger
-from operator import itemgetter
+
+from .utils import UTC, naive_dt_to_utc, now
 
 DEFAULT_PERF_LEVEL = 0  # 0: disabled; 1: enabled
 DEFAULT_PERF_PERIOD = 60
@@ -79,14 +80,23 @@ class PerformanceMetricsSettings:
 class Trace(object):
     """Trace execution time."""
 
-    def __init__(self, name=None, recorder=None):
+    def __init__(self, name=None, recorder=None, at=None):
         self.name = name
         self.recorder = recorder
         # TODO use a monotonic clock but there is none on Python < 3.3.
         # Maybe work around this with a native extension.
-        self.start_time = time.time()
+        if at is None:
+            self.start_time = time.time()
+        else:
+            at = naive_dt_to_utc(at)
+            self.start_time = (at - datetime(1970, 1, 1, tzinfo=UTC)).total_seconds()
         self.duration = None
 
+    @property
+    def start_at(self):
+        """Get the datetime associated with the start time."""
+        return datetime.fromtimestamp(self.start_time, tz=UTC)
+
     @property
     def duration_ms(self):
         """Convert the duration from seconds to milliseconds."""
@@ -105,15 +115,18 @@ class Trace(object):
     def end(self, at=None):
         """End a performance trace."""
         if self.duration is not None:
-            LOGGER.debug("A trace was ended multiple times: %r", self)
             return
-        end_time = time.time()
-        self.duration = end_time - (self.start_time or end_time)
+        if at is None:
+            end_time = time.time()
+        else:
+            at = naive_dt_to_utc(at)
+            end_time = (at - datetime(1970, 1, 1, tzinfo=UTC)).total_seconds()
+        self.duration = end_time - self.start_time
         if self.duration < 0:
             self.duration = 0
         if self.recorder is not None:
             if at is None:
-                at = datetime.utcnow()
+                at = datetime.fromtimestamp(end_time, tz=UTC)
             self.recorder.record_trace(self.name, self.start_time, self.duration_ms, at)
 
     def __repr__(self):
@@ -124,6 +137,36 @@ class Trace(object):
         )
 
 
+class AggregatedDuration:
+    """Compute total duration of potentially overlapping traces.
+    """
+
+    def __init__(self):
+        self._start_time = 0
+        self._counter = 0
+        self._duration = 0
+
+    def push_start(self, current_time):  # type: (float) -> None
+        if self._counter == 0:
+            self._start_time = current_time
+            self._counter = 1
+        else:
+            self._counter += 1
+
+    def push_stop(self, current_time):  # type: (float) -> None
+        if self._counter == 1:
+            self._duration += current_time - self._start_time
+            self._counter = 0
+        elif self._counter > 0:
+            self._counter -= 1
+
+    def get_duration(self, current_time):  # type: (float) -> float
+        if self._counter > 0:
+            # Still active spans, compute duration with current time
+            return self._duration + (current_time - self._start_time)
+        return self._duration
+
+
 class RequestTrace(Trace):
     """Trace request execution time for performance monitoring.
 
@@ -133,56 +176,27 @@ class RequestTrace(Trace):
     meaning when the end method is called, all sub-traces are ended too.
     """
 
-    def __init__(self, recorder=None):
+    def __init__(self, recorder=None, at=None):
         self._traces = []
-        self._timeline = []
-        super(RequestTrace, self).__init__("req", recorder)
+        self._aggregated_duration = AggregatedDuration()
+        super(RequestTrace, self).__init__("req", recorder, at=at)
 
-    def trace(self, name=None):
+    def trace(self, name=None, at=None):
         """Start a sub-trace."""
-        trace = Trace(name, recorder=self)
-        # Keep an history of all traces to compute total sub trace duration
-        self._timeline.append((1, (trace.start_time - self.start_time) * 1000.0))
+        trace = Trace(name, at=at, recorder=self)
+        self._aggregated_duration.push_start((trace.start_time - self.start_time) * 1000.0)
         self._traces.append(trace)
         return trace
 
-    def _compute_aggregated_duration(self):
-        # This is a modified version of the algorithm used to solve the
-        # "minimum number of platforms required" problem.
-        # It computes the overall time spent in sub-traces taking into
-        # account the overlapping traces.
-        counter = 0
-        start_t = 0
-        end_t = 0
-        duration = 0
-        # The timeline should be already in order, make sure it is
-        self._timeline.sort(key=itemgetter(1))
-        for op, t in self._timeline:
-            # Start interval
-            if counter == 0 and op == 1:
-                start_t = t
-            # End interval
-            elif counter == 1 and op == -1:
-                duration += t - start_t
-                end_t = t
-            elif op == -1:
-                end_t = t
-            counter += op
-        # If some traces are not finished, use the last interval end
-        if counter > 0:
-            duration += end_t - start_t
-        return duration if duration >= 0 else 0
-
     @property
     def aggregated_duration_ms(self):
         """Return total time spent in sub-traces."""
-        return self._compute_aggregated_duration()
+        duration = self._aggregated_duration.get_duration((time.time() - self.start_time) * 1000.0)
+        return duration if duration > 0 else 0
 
     def record_trace(self, name, start_time, duration, at):
         """Aggregate values from sub-traces."""
-        # End of a sub-trace, keep it in the timeline to compute aggregated
-        # duration
-        self._timeline.append((-1, (start_time - self.start_time) * 1000.0 + duration))
+        self._aggregated_duration.push_stop((start_time - self.start_time) * 1000.0 + duration)
         if self.recorder is not None:
             self.recorder.record_trace(name, start_time, duration, at)
 
@@ -190,15 +204,16 @@ class RequestTrace(Trace):
         """End the trace and all sub traces."""
         # Automatically terminate sub traces to assure consistency
         if at is None:
-            at = datetime.utcnow()
+            at = now()
         for trace in self._traces:
             if trace.duration is None:
                 trace.end(at=at)
         super(RequestTrace, self).end(at=at)
         if self.recorder is not None:
-            aggregated_duration_ms = self._compute_aggregated_duration()
+            duration_ms = self.duration_ms
+            aggregated_duration_ms = self._aggregated_duration.get_duration(duration_ms)
             self.recorder.record_trace("sq", None, aggregated_duration_ms, at)
-            no_sq = self.duration_ms - aggregated_duration_ms
+            no_sq = duration_ms - aggregated_duration_ms
             if no_sq > 0:
                 fraction = self.aggregated_duration_ms / no_sq
                 self.recorder.record_trace("pct", None, 100.0 * fraction, at)
