@@ -11,9 +11,10 @@ from functools import wraps
 
 from .condition_evaluator import ConditionEvaluator, is_condition_empty
 from .remote_exception import raw_traceback_formatter, traceback_formatter
+from .rules_actions.utils import retain_datadog_trace
 from .runtime_storage import runtime
 from .samplers import ScopedTracingSampler
-from .utils import get_datadog_correlation_ids, now
+from .utils import now
 
 LOGGER = logging.getLogger(__name__)
 
@@ -192,7 +193,7 @@ class BaseCallback(object):
         if payload_sections is None:
             payload_sections = self.DEFAULT_EXCEPTION_SECTIONS
         if "datadog-correlation-ids" in payload_sections:
-            trace_id, span_id = get_datadog_correlation_ids()
+            trace_id, span_id = retain_datadog_trace()
             payload["datadog_trace_id"] = trace_id
             payload["datadog_span_id"] = span_id
         self.storage.observe("sqreen_exceptions", payload,
