@@ -36,6 +36,7 @@ from .headers_insert import HeadersInsertCB
 from .headers_insert_django import HeadersInsertCBDjango
 from .headers_insert_flask import HeadersInsertCBFlask
 from .headers_insert_pyramid import HeadersInsertCBPyramid
+from .headers_transform import HeadersTransform
 from .ip_blacklist import IPBlacklistCB
 from .not_found import NotFoundCB
 from .reactive_log import ReactiveLog
@@ -51,7 +52,6 @@ from .record_request_pyramid import RecordRequestPyramid
 from .record_response import RecordResponse
 from .reflected_xss import ReflectedXSSCB
 from .regexp_rule import RegexpRule
-from .sdk_identify import SDKIdentify
 from .security_responses import (
     IPActionCB,
     SecurityResponsesIP,
@@ -61,6 +61,7 @@ from .shell_env import ShellEnvCB
 from .sqreen_error_page import SqreenErrorPage
 from .sqreen_error_page_django import SqreenErrorPageDjango
 from .sqreen_error_page_django_new_style import SqreenErrorPageDjangoNewStyle
+from .sqreen_error_page_falcon import SqreenErrorPageFalcon
 from .sqreen_error_page_flask import SqreenErrorPageFlask
 from .sqreen_error_page_pyramid import SqreenErrorPagePyramid
 from .user_agent_matches import UserAgentMatchesCB
@@ -68,6 +69,13 @@ from .user_agent_matches_framework import (
     UserAgentMatchesCBDjango,
     UserAgentMatchesCBFramework,
 )
+from .user_monitoring import (
+    DjangoAuthTrack,
+    SDKAuthTrack,
+    SDKIdentify,
+    SDKSignupTrack,
+    SDKTrackEvent,
+)
 from .waf import WAFCB
 from .wsgi_input_preview import WSGIInputPreview
 
@@ -78,9 +86,9 @@ except ImportError:
     pass
 
 if HAS_ASYNCIO:
-    from .record_request_context_aiohttp import RecordRequestContextAioHTTP
-    from .headers_insert_aiohttp import HeadersInsertCBAioHTTP
     from .count_http_codes_aiohttp import CountHTTPCodesCBAioHTTP
+    from .headers_insert_aiohttp import HeadersInsertCBAioHTTP
+    from .record_request_context_aiohttp import RecordRequestContextAioHTTP
     from .sqreen_error_page_aiohttp import SqreenErrorPageAioHTTP
 
 
@@ -155,6 +163,7 @@ __all__ = [
     "CountHTTPCodesCBPHP",
     "CountHTTPCodesCBPyramid",
     "CrawlerUserAgentMatchesMetricsCB",
+    "DjangoAuthTrack",
     "DjangoAuthMetrics",
     "ExtensionPassthruCB",
     "HeadersInsertCB",
@@ -162,6 +171,7 @@ __all__ = [
     "HeadersInsertCBDjango",
     "HeadersInsertCBFlask",
     "HeadersInsertCBPyramid",
+    "HeadersTransform",
     "IPActionCB",
     "IPBlacklistCB",
     "JSCB",
@@ -182,9 +192,12 @@ __all__ = [
     "RecordResponse",
     "ReflectedXSSCB",
     "RegexpRule",
+    "SDKAuthTrack",
     "SDKAuthMetrics",
     "SDKIdentify",
+    "SDKSignupTrack",
     "SDKSignupMetrics",
+    "SDKTrackEvent",
     "SecurityResponsesIP",
     "SecurityResponsesUser",
     "ShellEnvCB",
@@ -192,6 +205,7 @@ __all__ = [
     "SqreenErrorPageAioHTTP",
     "SqreenErrorPageDjango",
     "SqreenErrorPageDjangoNewStyle",
+    "SqreenErrorPageFalcon",
     "SqreenErrorPageFlask",
     "SqreenErrorPagePyramid",
     "UserAgentMatchesCB",
