@@ -9,18 +9,19 @@
 import logging
 
 from ..rules import RuleCallback
+from ..rules_actions import RecordObservationMixin
 
 LOGGER = logging.getLogger(__name__)
 
 
-class CountHTTPCodesCBDjango(RuleCallback):
+class CountHTTPCodesCBDjango(RecordObservationMixin, RuleCallback):
 
     INTERRUPTIBLE = False
 
-    def post(self, instance, args, kwargs, result=None, **options):
+    def post(self, instance, args, kwargs, options):
         """ Recover the status code and update the http_code metric
         """
-        response = result
+        response = options.get("result")
         status_code = response.status_code
         self.record_observation("http_code", str(status_code), 1)
 
