@@ -10,7 +10,7 @@ import logging
 
 from ..exceptions import InvalidArgument
 from ..rules import RuleCallback
-from ..utils import Mapping
+from ..utils import Mapping, cached_property, itervalues
 
 LOGGER = logging.getLogger(__name__)
 
@@ -25,7 +25,7 @@ def convert_to_str(headers):
         header_value = str(
             header_value.encode("latin-1", errors="replace").decode("latin-1")
         )
-        yield (header_name, header_value)
+        yield header_name, header_value
 
 
 class BaseHeadersInsertCB(RuleCallback):
@@ -39,40 +39,30 @@ class BaseHeadersInsertCB(RuleCallback):
             msg = "Invalid data type received: {}"
             raise InvalidArgument(msg.format(type(self.data)))
 
-        try:
-            self.values = self.data["values"]
-        except KeyError:
-            msg = "No key 'values' in data (had {})"
-            raise InvalidArgument(msg.format(self.data.keys()))
-
-        self._headers = None
-
-    @property
+    @cached_property
     def headers(self):
-        """ Cached property to defer headers data conversio,n
-        """
-        if self._headers is None:
-            self._headers = dict(convert_to_str(self.values))
-        return self._headers
+        return {
+            name.lower(): (name, value)
+            for name, value in convert_to_str(self.data.get("values", []))
+        }
 
 
 class HeadersInsertCB(BaseHeadersInsertCB):
     """ Callback that add the custom sqreen header in WSGI
     """
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         new_args = list(args)
         headers_to_add = dict(self.headers)
         new_headers = []
         current_headers = args[1]
-        for item in current_headers:
-            header = headers_to_add.pop(item[0], None)
-            if header is not None:
-                new_headers.append((item[0], header))
-            else:
-                new_headers.append(item)
-        for item in headers_to_add.items():
-            new_headers.append(item)
+
+        # Replace current headers
+        new_headers.extend([headers_to_add.pop(n.lower(), (n, v))
+                            for n, v in current_headers])
+
+        # Add remaining headers
+        new_headers.extend(itervalues(headers_to_add))
 
         new_args[1] = new_headers
         return {"status": "modify_args", "args": [new_args, kwargs]}
