@@ -14,15 +14,15 @@ LOGGER = logging.getLogger(__name__)
 
 
 class HeadersInsertCBDjango(BaseHeadersInsertCB):
-    """ Callback that add the custom sqreen header
+    """ Legacy callback that add the custom sqreen header
     """
 
-    def post(self, instance, args, kwargs, result=None, **options):
+    def post(self, instance, args, kwargs, options):
         """ Set headers
         """
         try:
-            response = result
-            for header_name, header_value in self.headers.items():
+            response = options.get("result")
+            for header_name, header_value in self.headers.values():
                 response[header_name] = header_value
         except Exception:
             LOGGER.warning("An error occurred", exc_info=True)
