@@ -15,16 +15,18 @@ from py_mini_racer.py_mini_racer import (  # type: ignore
 )
 
 from ..binding_accessor import BindingAccessor
-from ..constants import LIFECYCLE_METHODS
 from ..exceptions import SqreenException
 from ..rules import RuleCallback
+from ..rules_actions import (
+    RecordAttackMixin,
+    RecordObservationMixin,
+    RecordSignalMixin,
+    RequestStoreMixin,
+)
 from ..utils import (
     ALL_STRING_CLASS,
     CustomJSONEncoder,
     create_bound_method,
-    datetime_from_isoformat,
-    is_unicode,
-    naive_dt_to_utc,
     to_unicode_safe,
 )
 from .mini_racer_pool import JSContextPool
@@ -45,7 +47,7 @@ class JSException(SqreenException):
         return {"cb": self.callback, "args": self.arguments}
 
 
-class JSCB(RuleCallback):
+class JSCB(RecordObservationMixin, RecordAttackMixin, RecordSignalMixin, RequestStoreMixin, RuleCallback):
     """ A callback that run a JS function as pre / post / failing through
     py_mini_racer context.
     """
@@ -56,7 +58,7 @@ class JSCB(RuleCallback):
         # Create pre / post / failing callback methods on the instance early on
         # because RuleCallback.__init__ needs them.
         rule_callbacks = frozenset((kwargs.get("callbacks") or {}).keys())
-        for callback_name in LIFECYCLE_METHODS.values():
+        for callback_name in ("pre", "post", "failing",):
             if callback_name in rule_callbacks:
                 self.create_callback(callback_name)
 
@@ -64,14 +66,14 @@ class JSCB(RuleCallback):
 
         self._loaded = False
         self.arguments = {}
-        self.source = ''
+        self.source = u''
         self.code_id = None
 
     def create_callback(self, name):
         """ Create an execute callback alias called `name`.
         """
-        def callback(self, instance, args, kwargs, **options):
-            return self.execute(name, instance, args, kwargs, **options)
+        def callback(self, instance, args, kwargs, options):
+            return self.execute(name, instance, args, kwargs, options)
 
         try:
             callback.__name__ = name
@@ -85,24 +87,24 @@ class JSCB(RuleCallback):
         """ Create the binding accessors and save the js definitions
         """
 
-        self.source = ''
+        self.source = u''
         if self.callbacks:
             for callback_name, callback_args in self.callbacks.items():
 
                 if not isinstance(callback_args, list):
-                    self.source += "{} = {}\n".format(callback_name, callback_args)
+                    self.source += u"{} = {}\n".format(callback_name, to_unicode_safe(callback_args))
                     self.arguments[callback_name] = []
                 else:
-                    self.source += "{} = {}\n".format(callback_name, callback_args[-1])
+                    self.source += u"{} = {}\n".format(callback_name, to_unicode_safe(callback_args[-1]))
                     arguments = callback_args[:-1]
                     self.arguments[callback_name] = [
                         BindingAccessor(arg) for arg in arguments
                     ]
 
-        self.code_id = hashlib.md5(self.source.encode('utf-8')).hexdigest()
+        self.code_id = to_unicode_safe(hashlib.md5(self.source.encode('utf-8')).hexdigest())
         self._loaded = True
 
-    def execute(self, name, instance, args, kwargs, **options):
+    def execute(self, name, instance, args, kwargs, options):
         """ Fetches a context before executing _execute
         """
         if self._loaded is False:
@@ -117,16 +119,17 @@ class JSCB(RuleCallback):
         return pool.with_context(
             lambda ctx: self._execute(
                 ctx, name, budget_in_ms or 0, self.arguments[name],
-                instance, args, kwargs, **options))
+                instance, args, kwargs, options))
 
     def _execute(self, js_ctx, name, budget_in_ms, arguments,
-                 instance, args, kwargs, result=None, exc_info=None,
-                 **options):
+                 instance, args, kwargs, options):
         """ Execute a JS callback passed in definition.
         Handle recording attack, observations and chaining.
         Protected against infinite recursion with a max number of JS calls
         set to 100.
         """
+        exc_info = options.get("exc_info")
+        result = options.get("result")
 
         if self.code_id in js_ctx.failed_code_ids:
             LOGGER.debug("Skipping execution of callback %s (code md5 %s)"
@@ -158,6 +161,8 @@ class JSCB(RuleCallback):
                 "kwargs": kwargs,
                 "data": self.data,
                 "rv": return_value,
+                "request_store": self.storage.get_request_store(),
+                "options": options,
             }
 
             resolved_args = [
@@ -169,7 +174,7 @@ class JSCB(RuleCallback):
             LOGGER.debug("Resolved args %s for %s", resolved_args, arguments)
 
             try:
-                callee = "sqreen_data['{}']['{}']".format(self.code_id, name)
+                callee = u"sqreen_data['{}']['{}']".format(self.code_id, name)
                 result = js_ctx.call(
                     callee, *resolved_args,
                     encoder=CustomJSONEncoder, timeout=int(budget_in_ms)
@@ -185,14 +190,12 @@ class JSCB(RuleCallback):
             if result is None:
                 return result
 
-            # Process the return value
-            self._record_attack(result)
-            self._record_observations(result)
-
             # Check for chaining
             if result.get("call") is None:
                 return result
 
+            result = self.action_processor(result, options)
+
             # Prepare next call
             name = result["call"]
 
@@ -208,24 +211,6 @@ class JSCB(RuleCallback):
             else:
                 arguments = self.arguments[name]
 
-    def _record_attack(self, return_value):
-        """ Record an attack if the JS callback returned a record info
-        """
-        if return_value.get("record"):
-            self.record_attack(return_value["record"])
-
-    def _record_observations(self, return_value):
-        """ Record observations if the JS callback returned a observations list
-        """
-        if return_value.get("observations"):
-            for observation in return_value["observations"]:
-                if len(observation) > 3:
-                    at = observation[3]
-                    if is_unicode(at):
-                        at = datetime_from_isoformat(at)
-                    observation[3] = naive_dt_to_utc(at)
-                self.record_observation(*observation)
-
     def _restrict(self, name, arguments):
         """ Filter out useless values from iterables present in *arguments*.
         It only supports conditions with a single hash_val_include.
