@@ -24,7 +24,7 @@ class ReactiveWAF(WAFCBMixin, ReactiveRuleCallback):
         self.batch_addresses = frozenset(itertools.chain.from_iterable(
             values.get("rule_request", [])))
 
-    def handler(self, instance, args, kwargs, **options):
+    def handler(self, instance, args, kwargs, options):
         budget = self.get_remaining_budget(options)
         if budget is None:
             budget = self.max_budget_ms
