@@ -24,8 +24,8 @@ class RecordRequestContext(RecordRequest):
                 self.runner.queue, self.runner.observation_queue
             )
 
-    def post(self, instance, args, kwargs, **options):
+    def post(self, instance, args, kwargs, options):
         self._clear_request()
 
-    def failing(self, instance, args, kwargs, **options):
+    def failing(self, instance, args, kwargs, options):
         self._clear_request()
