@@ -15,9 +15,9 @@ LOGGER = getLogger(__name__)
 
 
 class RecordRequestContextPyramid(RecordRequestContext):
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         self._store_request(PyramidRequest(args[0]))
 
-    def post(self, instance, args, kwargs, result=None, **options):
-        self.storage.store_response(PyramidResponse(result))
+    def post(self, instance, args, kwargs, options):
+        self.storage.store_response(PyramidResponse(options.get("result")))
         self._clear_request()
