@@ -16,8 +16,8 @@ LOGGER = getLogger(__name__)
 
 class RecordRequestDjango(RecordRequest):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         self._store_request(DjangoRequest(args[0]))
 
-    def post(self, instance, args, kwargs, result=None, **options):
-        self.storage.store_response(DjangoResponse(result))
+    def post(self, instance, args, kwargs, options):
+        self.storage.store_response(DjangoResponse(options.get("result")))
