@@ -15,8 +15,8 @@ LOGGER = getLogger(__name__)
 
 
 class RecordRequestPyramid(RecordRequest):
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         self._store_request(PyramidRequest(args[0]))
 
-    def post(self, instance, args, kwargs, result=None, **options):
-        self.storage.store_response(PyramidResponse(result))
+    def post(self, instance, args, kwargs, options):
+        self.storage.store_response(PyramidResponse(options.get("result")))
