@@ -12,7 +12,7 @@ from .sqreen_error_page import BaseSqreenErrorPage
 
 class SqreenErrorPageFlask(BaseSqreenErrorPage):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
 
         exception = args[0]
         ret = self.handle_exception(exception)
