@@ -7,6 +7,7 @@
 """ Main runner module
 """
 
+import itertools
 import logging
 import sys
 from time import time
@@ -19,12 +20,11 @@ from .metrics import UnregisteredMetric
 from .performance_metrics import PerformanceMetricsSettings
 from .samplers import TracingSampler
 from .utils import now
-from .workflow import Workflow
 
 if sys.version_info[0] < 3:
-    from Queue import Empty, Queue, Full
+    from Queue import Empty, Full, Queue
 else:
-    from queue import Empty, Queue, Full
+    from queue import Empty, Full, Queue
 
 LOGGER = logging.getLogger(__name__)
 
@@ -50,49 +50,76 @@ class CappedQueue(object):
         self.maxsize = maxsize
         self.queue = Queue(self.maxsize)
         self.name = name or "queue"
+        self._count_ingress = itertools.count()
+        self._count_dropped = itertools.count()
+        self._count_egress = itertools.count()
+        self._last_count_ingress = 0
+        self._last_count_dropped = 0
+        self._last_count_egress = 0
 
     def get(self, timeout, block=True):
         """ Wait for up to timeout for an item to return and block while
         waiting
         """
-        return self.queue.get(timeout=timeout, block=block)
+        ret = self.queue.get(timeout=timeout, block=block)
+        next(self._count_egress)
+        return ret
 
     def get_nowait(self):
         """ Get without waiting, raise queue.Empty if nothing is present
         """
-        return self.queue.get_nowait()
+        ret = self.queue.get_nowait()
+        next(self._count_egress)
+        return ret
 
     def put(self, item):
         """ Tries to put an item to the queue, if the queue is full, pop an
         item and try again
         """
-        pushed = False
-        dropped = 0
-        while pushed is False:
+        while True:
             try:
-                self.queue.put_nowait(item)
-                pushed = True
+                ret = self.queue.put_nowait(item)
+                next(self._count_ingress)
+                return ret
             except Full:
                 try:
                     self.queue.get_nowait()
                     self.queue.task_done()
-                    dropped += 1
+                    next(self._count_dropped)
                 except Empty:
                     pass
-        if dropped:
-            LOGGER.error("%d items dropped from the %s", dropped, self.name)
-
-    def clear(self):
-        """ Clear all items in the queue
-        """
-        with self.queue.mutex:
-            self.queue.queue.clear()
 
     def half_full(self):
         """ Return True if the current queue size if at least half the maxsize
         """
         return self.queue.qsize() > (self.maxsize / 2.)
 
+    def get_metrics(self, at=None):
+        """ Return the queue metrics, this method is *NOT* thread-safe.
+        """
+        ingress_delta = next(self._count_ingress) - self._last_count_ingress
+        dropped_delta = next(self._count_dropped) - self._last_count_dropped
+        egress_delta = next(self._count_egress) - self._last_count_egress
+        self._last_count_ingress += ingress_delta + 1
+        self._last_count_dropped += dropped_delta + 1
+        self._last_count_egress += egress_delta + 1
+        at = at or now()
+        return [
+            ("event_manager", at, "{}_ingress".format(self.name), ingress_delta),
+            ("event_manager", at, "{}_dropped".format(self.name), dropped_delta),
+            ("event_manager", at, "{}_egress".format(self.name), egress_delta),
+        ]
+
+    def task_done(self):
+        """ Inform the queue that an event was processed.
+        """
+        self.queue.task_done()
+
+    def join(self):
+        """ Block until all events are consumed
+        """
+        self.queue.join()
+
 
 class RunnerSettings(object):
     """ Various values that need to be shared across threads
@@ -163,16 +190,27 @@ def process_initial_commands(initial_payload, runner):
     """
     commands = initial_payload.get("commands", [])
     rulespack_id = initial_payload.get("pack_id", None)
-    rules = initial_payload.get("rules", None)
+    rules = initial_payload.get("rules", [])
 
     # Preprocess commands first
     for command in commands:
-        if command["name"] == "instrumentation_enable" and not command["params"]:
-            command["params"] = (rulespack_id, rules)
+        if command["name"] == "instrumentation_enable":
+            params = command.get("params")
+            if not params:
+                command["params"] = (rulespack_id, rules)
+            break
+    else:
+        # Always instrument even without a rulespack
+        # in fact, some frameworks require a very early
+        # instrumentation and cannot wait the next heartbeat.
+        commands.append({
+            "uuid": "static-instrumentation-enable",
+            "name": "instrumentation_enable",
+            "params": (rulespack_id, rules),
+        })
 
     # Process commands normally
-    if commands:
-        runner.process_commands(commands)
+    runner.process_commands(commands)
 
     actions = initial_payload.get("actions")
     runner.action_store.reload_from_dicts(actions or [])
@@ -250,6 +288,10 @@ class Runner(BaseRunner):
             PerformanceMetricsSettings.from_features(initial_features)
         )
 
+        self.set_event_manager_metrics_period(
+            initial_features.get("event_manager_metrics_period", 60)
+        )
+
         self.budget = None
 
         self.set_whitelisted_metric(initial_features.get("whitelisted_metric", True))
@@ -264,53 +306,52 @@ class Runner(BaseRunner):
         # Next things to send on next heartbeat
         self.next_commands_results = {}
 
-        if self.interface_manager is not None:
-            self.interface_manager.register(Workflow(self))
-
     def run(self):
         """ Infinite loop
         """
-        self.logger.debug("Starting the runner (%s)",
-                          "with instrumentation enabled" if self.instrumentation.enabled
-                          else "waiting for instrumentation")
+        self.logger.debug("Starting the runner")
         while self.stop is False:
             self.run_once()
         self.logger.debug("Exiting the runner now")
 
-    def run_once(self, block=True):
-        """ Tries to pop a message or send an heartbeat
+    def handle_messages(self, block=True):
+        """ Tries to pop and handle most messages before the heartbeat delay
         """
-        try:
-            event = self.queue.get(timeout=self.sleep_delay, block=block)
-            self.logger.debug(
-                "Run once, event: %s after %ss sleep delay",
-                event,
-                self.sleep_delay,
-            )
-            self.handle_message(event)
+        timeout = self.heartbeat_delay
+        deadline = time() + timeout
+        while timeout > 0:
+            try:
+                event = self.queue.get(timeout=timeout, block=block)
+                self.handle_message(event)
+                self.queue.task_done()
+                # Exit now if should stop
+                if self.stop:
+                    return
+            except Empty:
+                # timeout exceeded or not blocking
+                break
+            finally:
+                # Tick the deliverer to publish batch if necessary
+                self.deliverer.tick()
 
-            # Exit now if should stop
-            if self.stop:
-                return
-        except Empty:
-            self.logger.debug(
-                "No message after %ss sleep delay", self.sleep_delay
-            )
+            # Compute remaining time
+            timeout = deadline - time()
 
+    def run_once(self):
+        """ Tries to handle most messages and send an heartbeat after a given delay
+        """
+        self.handle_messages(block=True)
+        if self.stop:
+            return
+        self.logger.debug("Heartbeat after %ss delay", self.heartbeat_delay)
         # Aggregate observations in transit in observations queue
         self.aggregate_observations()
-
-        if self._should_do_heartbeat():
-            self.do_heartbeat()
-
-        # Tick the deliverer to publish batch if necessary
-        self.deliverer.tick()
-
-    @property
-    def sleep_delay(self):
-        """ Compute the sleeping delay by taking the heartbeat
-        """
-        return self.heartbeat_delay
+        # Publish the metrics if signals are enabled otherwise they are
+        # sent as part of the heartbeat
+        if self.session.use_signals:
+            self.publish_metrics(force_finalize=False)
+        # Always finish with an heartbeat
+        self.do_heartbeat()
 
     def handle_message(self, event):
         """ Handle incoming message
@@ -322,6 +363,7 @@ class Runner(BaseRunner):
         elif event is MetricsEvent:
             self.aggregate_observations()
         else:
+            self.logger.debug("Handle event: %r", event)
             self.deliverer.post_event(event)
 
     def process_commands(self, commands):
@@ -334,11 +376,10 @@ class Runner(BaseRunner):
         """ Do an heartbeat, publish finished metrics from MetricsStore and
         past commands results
         """
-        use_signals = self.session.use_signals
         payload = {
             "command_results": self.next_commands_results,
         }
-        if not use_signals:
+        if not self.session.use_signals:
             metrics = self.metrics_store.get_data_to_publish(
                 now(), force_finalize=False)
             payload["metrics"] = [
@@ -355,9 +396,6 @@ class Runner(BaseRunner):
         if res is not None:
             self.process_commands(res["commands"])
 
-        if use_signals:
-            self.publish_metrics(force_finalize=False)
-
     def aggregate_observations(self):
         """ Empty the observation queue and update the metric store
         with the observations in the queue.
@@ -372,6 +410,19 @@ class Runner(BaseRunner):
                     unregistered_metrics.add(name)
         except Empty:
             pass
+
+        if self.event_manager_metrics_period:
+            at = now()
+            try:
+                for name, item_at, key, value in itertools.chain(
+                        self.queue.get_metrics(at=at),
+                        self.observation_queue.get_metrics(at=at),
+                        self.session.get_metrics(at=at)):
+                    if value:
+                        self.metrics_store.update(name, item_at, key, value)
+            except UnregisteredMetric:
+                unregistered_metrics.add(name)
+
         if unregistered_metrics:
             # metrics could be have been queued before
             # they were disabled by a feature change
@@ -386,11 +437,6 @@ class Runner(BaseRunner):
         if metrics:
             self.deliverer.post_metrics(metrics)
 
-    def _should_do_heartbeat(self):
-        """ Check if we should send an heartbeat because the delay is overdue
-        """
-        return (self.last_heartbeat_request + self.heartbeat_delay) < time()
-
     def logout(self):
         """ Run cleanup
         """
@@ -424,6 +470,7 @@ class Runner(BaseRunner):
                 "sqreen_call_counts"
             ),
             "whitelisted_metric": self.whitelisted_metric,
+            "event_manager_metrics_period": self.event_manager_metrics_period,
         }
         res.update(self.performance_metrics_settings.as_features())
         return res
@@ -497,6 +544,13 @@ class Runner(BaseRunner):
         else:
             self.metrics_store.unregister_metric_definition("whitelisted")
 
+    def set_event_manager_metrics_period(self, period):
+        self.event_manager_metrics_period = period
+        if period > 0:
+            self.metrics_store.register_metric_definition("event_manager", "Sum", period)
+        else:
+            self.metrics_store.unregister_metric_definition("event_manager")
+
     def set_deliverer(self, batch_size, max_staleness):
         # Drain current deliverer
         self.deliverer.drain(resiliently=True)
