@@ -681,7 +681,7 @@ class RunnerThread(Thread):
             logger = getLogger(__name__)
             logger.warning(GRACEFUL_FAIL_MSG, CONFIG["GRACEFUL_TIMEOUT"])
         else:
-            if self.isAlive():
+            if self.is_alive():
                 logger = getLogger(__name__)
                 logger.warning(GRACEFUL_FAIL_MSG, CONFIG["GRACEFUL_TIMEOUT"])
 
