@@ -85,7 +85,7 @@ def parse_minor_version(version):
 
 
 SUPPORTED_PYTHON_VERSIONS = {(2, 7), (3, 4), (3, 5), (3, 6), (3, 7), (3, 8),
-                             (3, 9)}
+                             (3, 9), (3, 10)}
 
 
 class RuntimeInfos(object):
