@@ -7,6 +7,7 @@
 """Helper classes for runtime storage."""
 
 import threading
+from collections import defaultdict
 
 import pkg_resources
 
@@ -87,14 +88,14 @@ class RuntimeStorage(object):
 
     def update_request_store(self, **data):
         """Update the request store with new data."""
-        store = self.store.setdefault("current_request_store", {})
+        store = self.store.setdefault("current_request_store", defaultdict(lambda: None))
         if len(store) + len(data) > REQUEST_STORE_MAX_ITEMS:
             raise RequestStoreFull("request store capacity exceeded")
         store.update(data)
 
     def get_request_store(self):
         """Get the request store."""
-        return self.store.get("current_request_store", {})
+        return self.store.get("current_request_store", defaultdict(lambda: None))
 
     def store_response(self, response):
         """Store a response."""
