@@ -15,7 +15,7 @@ from ._vendors import rsa  # type: ignore
 from .utils import HAS_TYPING, is_string
 
 if HAS_TYPING:
-    from typing import Mapping, FrozenSet
+    from typing import FrozenSet, Mapping
 
 
 LOGGER = getLogger(__name__)
