@@ -596,17 +596,6 @@ class cached_property(property):
         return value
 
 
-def get_datadog_correlation_ids():
-    """Return the trace ID and span ID from dd-trace.
-    """
-    try:
-        from ddtrace import helpers
-
-        return helpers.get_correlation_ids()
-    except Exception:
-        return None, None
-
-
 def parse_form_params(qs):
     """Return a dict of form data after Basic HTTP form parsing."""
     qs = to_unicode_safe(qs).replace("+", " ")
